* Citation: Yi-Yi Chen, Peter Schmidt, Hung-Jen Wang (2014) "Consistent estimation of the
* fixed effects stochastic frontier model", Journal of Econometrics

* The script estimates the following models: OLS, ALS (1976), and CSW (2014).
* Note 1: The required Stata programs (.ado) for estimating the CSW model
*   are included in the same zip file.
* Note 2: The script estimates the CSW using results from the ALS model as initial values.
* Note 3: To estimate the ASL model, you may have to install the Stata programs (free) by
*   following instructions on the following page.
*   http://homepage.ntu.edu.tw/~wangh/book-software.html

set trace off
set more off
set mem 20m
capture log close
log using demo_1.txt, text replace

use demo, clear

global yvar lny
global xvar llabor lfuel lk  llabor2 lfuel2 lk2  labfue2 labk2  fuek2 trend

*! ------ firm dummies -------

quie tab firm, gen(fid)
global dvar fid*

*! ------------------ OLS --------------

reg $yvar $xvar $dvar, nocons
mat bols = e(b)

*! ------------- ALS (1976), no fixed effect ---------

reg $yvar $xvar
mat b0 = e(b)

sfmodel $yvar, frontier($xvar) usigmas() vsigmas() prod dist(h)
ml init b0 0.1 0.1, copy
sf_srch, n(2) nograph fast
sf_srch, n(2) frontier($xvar) nograph fast
ml max, difficult gtol(1e-2) nrtol(1e-2) gradient
sf_predict, jlms(jlms_1)

sf_transform


*! ----------- ALS (1976), with dummies -------

sfmodel $yvar, frontier($xvar $dvar, nocons) usigmas()  vsigmas() prod dist(h)
ml init bols -0.1 -0.1, copy
sf_srch, frontier($xvar $dvar) n(2) fast
ml max, diff gradient gtol(1e-1) nrtol(1e-1)

sf_predict, jlms(jlms_2)
sf_transform

mat b1 = e(b)
mat b1a = b1[1, 1..10]
mat b1u = b1[1, "usigmas:"]
mat b1v = b1[1, "vsigmas:"]
scalar b1uc = b1u[1,1]
scalar b1vc = b1v[1,1]
scalar sig2u = exp(b1uc)
scalar sig2v = exp(b1vc)
local lambda_ini = ln(sqrt(sig2u/sig2v))
local sigma_ini = ln(sqrt(sig2u + sig2v))

*! ----------- CSW (2014 JoE), Within Estimate -------

global cham_pi firm
global cham_pt year
global cham_nd 64
scalar cham_nd = $cham_nd

/* for gauss-hermite integration */
mat noeuds=.
mat poids=.
qui ghquadm $cham_nd noeuds poids
mata: noeuds_m = st_matrix("noeuds")
mata: poids_m = st_matrix("poids")
mata: cham_nd = st_numscalar("cham_nd")

****************************

ml model d0 mle_within (frontier: $yvar = $xvar, nocons) (lambda: ) (sigma: ), technique(nr)
ml init  b1a  `lambda_ini'  `sigma_ini', copy

forvalues i = 1/2 {  /* could loop more, especially for lambda and sigma */
  ml_sf plot lambda:_cons
  ml_sf plot sigma:_cons
    foreach X in $xvar {
      ml_sf plot frontier:`X'
     }
}
forvalues i = 1/3 {
     ml_sf plot lambda:_cons
     ml_sf plot sigma:_cons
}

ml max, diff /* nonrtol  tol(1e-4) */

_diparm lambda sigma, label(sigma_v_sqr) prob ///
        function( (exp(2*@2)) / (1+exp(2*@1)) ) ///
        deriv(-(1+exp(2*@1))^(-2)*2*exp(2*@2+@1) 2*(1+exp(2*@1))^(-1)*exp(2*@2))

_diparm lambda sigma, label(sigma_u_sqr) prob ///
        function( exp(2*@2)*(exp(2*@1)/(1+exp(2*@1))) ) ///
        deriv(2*exp(2*(@1+@2))/((1+exp(2*@1))^2) 2*exp(2*(@1+@2))/(1+exp(2*@1)))

global lambda_within = [lambda]_b[_cons] /* do not change the global name; used in mle_between */
global sigma_within = [sigma]_b[_cons]   /* do not change the global name; used in mle_between */

scalar elambda   =  exp([lambda]_b[_cons]) /* sigma_u / sigma_v */
scalar esigma    =  exp([sigma]_b[_cons])  /* sqrt(sigma_u^2 + sigma_v^2 ) */
scalar esigma2   =  esigma^2               /* sigma_u^2 + sigma_v^2 */

scalar esigma2v = esigma^2/(1+elambda^2)
scalar esigma2u = esigma^2 - esigma2v

predict double fxb, xb equation(frontier)

   /* ---- alpha (mean method) and JLMS inefficiency index ---- */

capture drop epsilon musta sigs2 rat1

gen double ep2 = $yvar - fxb + sqrt(esigma2u)*sqrt(2/_pi)
bysort firm: egen alpha_mean = mean(ep2)

gen double epsilon = $yvar - fxb - alpha_mean
gen double musta = -(esigma2u*epsilon)/(esigma2u+esigma2v)
gen double sigs2 = (esigma2u*esigma2v)/(esigma2u+esigma2v)

gen double rat1 = musta/sqrt(sigs2)
gen double jlms_wm = (sqrt(sigs2)*normalden(rat1)/normal(rat1))+musta
